// import Base64 from "crypto-js/enc-base64";
import CryptoJS from "crypto-js";
import AudioPlayer from "./audioPlayer.esm";
import Base64 from "./base64";

export default class TtsVoice {
  appId = "";
  apiSecret = "";
  apiKey = "";
  ttsWS = null;
  audioPlayer = null;
  text = "";
  playing = false;
  running = false;
  done = false;
  error = false;
  errorMsg = "";
  dataFilled = false;

  constructor(params) {
    this.appId = params.appId;
    this.apiSecret = params.apiSecret;
    this.apiKey = params.apiKey;
    this.audioPlayer = new AudioPlayer();
    this.audioPlayer.onPlay = () => {
      this.playing = true;
    };
    this.audioPlayer.onStop = () => {
      // console.log(audioDatas);
      this.playing = false;
    };
  }

  setText(text) {
    this.text = text;
  }

  run(text) {
    if (!text) return;

    this.text = "";
    this.playing = false;
    this.done = false;
    this.error = false;
    this.errorMsg = "";
    this.dataFilled = false;

    this.text = text;
    this.running = true;
    this.connect();
  }

  stopRun() {
    this.ttsWS?.close();
    this.audioPlayer.reset();
    this.running = false;
    this.playing = false;
    this.done = false;
  }

  play() {
    if (!this.playing) this.audioPlayer.play();
  }
  stop() {
    if (this.playing) this.audioPlayer.stop();
  }

  getBlob() {
    return this.audioPlayer.getAudioDataBlob("mp3");
  }

  encodeText(text, type) {
    if (type === "unicode") {
      let buf = new ArrayBuffer(text.length * 4);
      let bufView = new Uint16Array(buf);
      for (let i = 0, strlen = text.length; i < strlen; i++) {
        bufView[i] = text.charCodeAt(i);
      }
      let binary = "";
      let bytes = new Uint8Array(buf);
      let len = bytes.byteLength;
      for (let i = 0; i < len; i++) {
        binary += String.fromCharCode(bytes[i]);
      }
      return window.btoa(binary);
    } else {
      return Base64.encode(text);
    }
  }
  getWebSocketUrl() {
    let apiKey = this.apiKey;
    let apiSecret = this.apiSecret;
    var url = "wss://tts-api.xfyun.cn/v2/tts";
    var host = location.host;
    var date = new Date().toGMTString();
    var algorithm = "hmac-sha256";
    var headers = "host date request-line";
    var signatureOrigin = `host: ${host}\ndate: ${date}\nGET /v2/tts HTTP/1.1`;
    var signatureSha = CryptoJS.HmacSHA256(signatureOrigin, apiSecret);
    var signature = CryptoJS.enc.Base64.stringify(signatureSha);
    var authorizationOrigin = `api_key="${apiKey}", algorithm="${algorithm}", headers="${headers}", signature="${signature}"`;
    var authorization = btoa(authorizationOrigin);
    url = `${url}?authorization=${authorization}&date=${date}&host=${host}`;
    return url;
  }

  connect() {
    const text = this.text;
    const appId = this.appId;
    const apiKey = this.apiKey;
    const apiSecret = this.apiSecret;
    const url = this.getWebSocketUrl(apiKey, apiSecret);
    if ("WebSocket" in window) {
      this.ttsWS = new WebSocket(url);
    } else if ("MozWebSocket" in window) {
      this.ttsWS = new window.MozWebSocket(url);
    } else {
      alert("浏览器不支持WebSocket");
      return;
    }

    const _this = this;
    this.ttsWS.onopen = () => {
      _this.audioPlayer.start({
        autoPlay: true,
        sampleRate: 16000,
        resumePlayDuration: 1000,
      });
      var tte = "UTF8";
      var params = {
        common: {
          app_id: appId,
        },
        business: {
          aue: "raw",
          auf: "audio/L16;rate=16000",
          vcn: "50",
          speed: 50,
          volume: 50,
          pitch: 50,
          bgs: 1,
          tte,
        },
        data: {
          status: 2,
          text: _this.encodeText(text, tte),
        },
      };
      _this.ttsWS.send(JSON.stringify(params));
    };

    this.ttsWS.onmessage = (e) => {
      let jsonData = JSON.parse(e.data);
      // 合成失败
      if (jsonData.code !== 0) {
        console.error(jsonData);
        this.stopConnect();
        this.error = true;
        this.errorMsg = jsonData.message;
        return;
      }
      // console.log(e);

      _this.dataFilled = true;
      _this.audioPlayer.postMessage({
        type: "base64",
        data: jsonData.data.audio,
        isLastData: jsonData.data.status === 2,
      });
      if (jsonData.code === 0 && jsonData.data.status === 2) {
        _this.ttsWS.close();
        setTimeout(() => {
          this.running = false;
          this.done = true;
        }, 300);
      }
    };
    _this.ttsWS.onerror = (e) => {
      console.error(e);
      this.stopConnect();
      this.error = true;
      this.errorMsg = e.message || "tts error";
    };
    _this.ttsWS.onclose = (e) => {
      console.log(e);
    };
  }

  stopConnect() {
    this.running = false;
    this.playing = false;
    this.ttsWS?.close();
    this.audioPlayer.reset();
    this.dataFilled = false;
  }

  download() {
    const blob = this.audioPlayer.getAudioDataBlob("mp3");
    if (!blob) {
      return;
    }
    let defaultName = new Date().getTime();
    let node = document.createElement("a");
    node.href = window.URL.createObjectURL(blob);
    node.download = `${defaultName}.mp3`;
    node.click();
    node.remove();
  }

  reset() {
    this.text = "";
    this.playing = false;
    this.running = false;
    this.done = false;
    this.error = false;
    this.errorMsg = "";
    this.dataFilled = false;
    this.ttsWS?.close();
    this.audioPlayer?.reset();
    this.ttsWS = null;
    this.audioPlayer = null;
  }
}
